-- SQL to create the initial tables for the MediaWiki database.
-- This is read and executed by the install script; you should
-- not have to run it by itself unless doing a manual install.

CREATE TABLE "user" (
  user_id		SERIAL NOT NULL PRIMARY KEY,
  user_name		VARCHAR(255) DEFAULT '' NOT NULL,
  user_real_name	VARCHAR(255) DEFAULT '',
  user_password		VARCHAR(128) DEFAULT '',
  user_newpassword	VARCHAR(128) default '',
  user_email		VARCHAR(255) default '',
  user_options		TEXT default '',
  user_touched		TIMESTAMP WITH TIME ZONE,
  user_token		CHAR(32) default '',
  user_email_authenticated TIMESTAMP WITH TIME ZONE DEFAULT NULL,
  user_email_token	CHAR(32),
  user_email_token_expires TIMESTAMP WITH TIME ZONE DEFAULT NULL
);
CREATE UNIQUE INDEX user_name_idx ON "user" (user_name);
CREATE INDEX user_email_token_idx ON "user" (user_email_token);

CREATE TABLE user_groups (
	ug_user		INTEGER NOT NULL
				REFERENCES "user" (user_id)
				ON DELETE CASCADE,
	ug_group	VARCHAR(16) NOT NULL,
	CONSTRAINT user_groups_pk PRIMARY KEY (ug_user, ug_group)
);
CREATE INDEX user_groups_group_idx ON user_groups(ug_group);

CREATE TABLE user_newtalk (
	user_id		INTEGER DEFAULT 0 NOT NULL,
	user_ip		VARCHAR(40) DEFAULT '' NOT NULL
);
CREATE INDEX user_newtalk_id_idx ON user_newtalk(user_id);
CREATE INDEX user_newtalk_ip_idx ON user_newtalk(user_ip);

CREATE TABLE page (
	page_id			SERIAL NOT NULL PRIMARY KEY,
	page_namespace		INTEGER NOT NULL,
	page_title		VARCHAR(255) NOT NULL,
	page_restrictions	TEXT DEFAULT '',
	page_counter 		INTEGER DEFAULT 0 NOT NULL,
	page_is_redirect	INTEGER DEFAULT 0 NOT NULL,
	page_is_new		INTEGER DEFAULT 0 NOT NULL,
	page_random		NUMERIC NOT NULL,
	page_touched		TIMESTAMP WITH TIME ZONE,
	page_latest		INTEGER NOT NULL,
	page_len 		INTEGER DEFAULT 0
);
CREATE UNIQUE INDEX page_id_namespace_title_idx ON page(page_namespace, page_title);
CREATE INDEX page_random_idx ON page(page_random);
CREATE INDEX page_len_idx ON page(page_len);

CREATE TABLE revision (
	rev_id		SERIAL NOT NULL,
	rev_page	INTEGER NOT NULL
				REFERENCES page (page_id)
				ON DELETE CASCADE,
	rev_text_id	INTEGER NOT NULL,
	rev_comment	TEXT,
	rev_user	INTEGER DEFAULT 0 NOT NULL,
	rev_user_text	VARCHAR(255) DEFAULT '' NOT NULL,
	rev_timestamp	TIMESTAMP WITH TIME ZONE NOT NULL,
	rev_minor_edit	INTEGER DEFAULT 0 NOT NULL,
	rev_deleted	INTEGER DEFAULT 0 NOT NULL,
	CONSTRAINT revision_pk PRIMARY KEY (rev_page, rev_id)
);

CREATE UNIQUE INDEX rev_id_idx ON revision(rev_id);
CREATE INDEX rev_timestamp_idx ON revision(rev_timestamp);
CREATE INDEX rev_page_timestamp_idx ON revision(rev_page, rev_timestamp);
CREATE INDEX rev_user_timestamp_idx ON revision(rev_user, rev_timestamp);
CREATE INDEX rev_usertext_timestamp_idx ON revision(rev_user_text, rev_timestamp);


CREATE TABLE text (
	old_id		SERIAL NOT NULL,
	old_text	TEXT,
	old_flags	TEXT,
	CONSTRAINT text_pk PRIMARY KEY (old_id)
);

CREATE TABLE archive (
	ar_namespace	INTEGER NOT NULL,
	ar_title	VARCHAR(255) NOT NULL,
	ar_text		TEXT,
	ar_comment	TEXT,
	ar_user		INTEGER,
	ar_user_text	VARCHAR(255) NOT NULL,
	ar_timestamp	TIMESTAMP WITH TIME ZONE NOT NULL,
	ar_minor_edit	INTEGER DEFAULT 0 NOT NULL,
	ar_flags	TEXT,
	ar_rev_id	INTEGER,
	ar_text_id	INTEGER
);
CREATE INDEX archive_name_title_timestamp ON archive(ar_namespace,ar_title,ar_timestamp);

CREATE TABLE pagelinks (
	pl_from	INTEGER NOT NULL
				REFERENCES page(page_id)
				ON DELETE CASCADE,
	pl_namespace	INTEGER DEFAULT 0 NOT NULL,
	pl_title	VARCHAR(255) NOT NULL
);
CREATE UNIQUE INDEX pl_from ON pagelinks(pl_from, pl_namespace, pl_title);
CREATE INDEX pl_namespace ON pagelinks(pl_namespace, pl_title);

CREATE TABLE imagelinks (
	il_from	 INTEGER NOT NULL REFERENCES page(page_id) ON DELETE CASCADE,
	il_to	 VARCHAR(255) NOT NULL
);
CREATE UNIQUE INDEX il_from ON imagelinks(il_from, il_to);
CREATE INDEX il_to ON imagelinks(il_to);

CREATE TABLE categorylinks (
  cl_from	INTEGER NOT NULL REFERENCES page(page_id) ON DELETE CASCADE,
  cl_to		VARCHAR(255) NOT NULL,
  cl_sortkey	VARCHAR(86) DEFAULT '',
  cl_timestamp	TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT now() 
);
CREATE UNIQUE INDEX cl_from ON categorylinks(cl_from, cl_to);
CREATE INDEX cl_sortkey ON categorylinks(cl_to, cl_sortkey);
CREATE INDEX cl_timestamp ON categorylinks(cl_to, cl_timestamp);

--
-- Contains a single row with some aggregate info
-- on the state of the site.
--
CREATE TABLE site_stats (
  ss_row_id		INTEGER NOT NULL,
  ss_total_views	INTEGER default 0,
  ss_total_edits	INTEGER default 0,
  ss_good_articles	INTEGER default 0,
  ss_total_pages	INTEGER default -1,
  ss_users		INTEGER default -1,
  ss_admins		INTEGER default -1
);
CREATE UNIQUE INDEX ss_row_id ON site_stats(ss_row_id);

--
-- Stores an ID for every time any article is visited;
-- depending on $wgHitcounterUpdateFreq, it is
-- periodically cleared and the page_counter column
-- in the page table updated for the all articles
-- that have been visited.)
--
CREATE TABLE hitcounter (
	hc_id	INTEGER NOT NULL
);

--
-- The internet is full of jerks, alas. Sometimes it's handy
-- to block a vandal or troll account.
--
CREATE TABLE ipblocks (
	ipb_id		SERIAL NOT NULL,
	ipb_address	VARCHAR(40),
	ipb_user	INTEGER,
	ipb_by		INTEGER NOT NULL
				REFERENCES "user" (user_id)
				ON DELETE CASCADE,
	ipb_reason	TEXT,
	ipb_timestamp	TIMESTAMP WITH TIME ZONE NOT NULL,
	ipb_auto	INTEGER DEFAULT 0 NOT NULL,
	ipb_expiry	TIMESTAMP WITH TIME ZONE,
	CONSTRAINT ipblocks_pk PRIMARY KEY (ipb_id)
);
CREATE INDEX ipb_address ON ipblocks(ipb_address);
CREATE INDEX ipb_user ON ipblocks(ipb_user);

CREATE TABLE image (
	img_name	VARCHAR(255) NOT NULL,
	img_size	INTEGER NOT NULL,
	img_width	INTEGER NOT NULL,
	img_height	INTEGER NOT NULL,
	img_metadata	TEXT,
	img_bits	INTEGER,
	img_media_type	VARCHAR(10),
	img_major_mime	VARCHAR(12) DEFAULT 'unknown',
	img_minor_mime	VARCHAR(32) DEFAULT 'unknown',
	img_description	TEXT NOT NULL,
	img_user	INTEGER NOT NULL REFERENCES "user"(user_id) ON DELETE CASCADE,
	img_user_text	VARCHAR(255) NOT NULL,
	img_timestamp	TIMESTAMP WITH TIME ZONE,
	CONSTRAINT image_pk PRIMARY KEY (img_name)
);
CREATE INDEX img_size_idx ON image(img_size);
CREATE INDEX img_timestamp_idx ON image(img_timestamp);

CREATE TABLE oldimage (
	oi_name		VARCHAR(255) NOT NULL,
	oi_archive_name	VARCHAR(255) NOT NULL,
	oi_size		INTEGER NOT NULL,
	oi_width	INTEGER NOT NULL,
	oi_height	INTEGER NOT NULL,
	oi_bits		INTEGER NOT NULL,
	oi_description	TEXT,
	oi_user		INTEGER NOT NULL REFERENCES "user"(user_id),
	oi_user_text	VARCHAR(255) NOT NULL,
	oi_timestamp	TIMESTAMP WITH TIME ZONE NOT NULL
);
CREATE INDEX oi_name ON oldimage (oi_name);

CREATE TABLE recentchanges (
	rc_id 		SERIAL NOT NULL,
	rc_timestamp	TIMESTAMP WITH TIME ZONE,
	rc_cur_time	TIMESTAMP WITH TIME ZONE,
	rc_user		INTEGER DEFAULT 0 NOT NULL,
	rc_user_text	VARCHAR(255),
	rc_namespace	INTEGER DEFAULT 0 NOT NULL,
	rc_title	VARCHAR(255) NOT NULL,
	rc_comment	VARCHAR(255),
	rc_minor	INTEGER DEFAULT 0 NOT NULL,
	rc_bot		INTEGER DEFAULT 0 NOT NULL,
	rc_new 		INTEGER DEFAULT 0 NOT NULL,
	rc_cur_id	INTEGER,
	rc_this_oldid	INTEGER NOT NULL,
	rc_last_oldid	INTEGER NOT NULL,
	rc_type		INTEGER DEFAULT 0 NOT NULL,
	rc_moved_to_ns	INTEGER,
	rc_moved_to_title	VARCHAR(255),
	rc_patrolled	INTEGER DEFAULT 0 NOT NULL,
	rc_ip		VARCHAR(40),
	CONSTRAINT rc_pk PRIMARY KEY (rc_id)
);
CREATE INDEX rc_timestamp ON recentchanges (rc_timestamp);
CREATE INDEX rc_namespace_title ON recentchanges(rc_namespace, rc_title);
CREATE INDEX rc_cur_id ON recentchanges(rc_cur_id);
CREATE INDEX new_name_timestamp ON recentchanges(rc_new, rc_namespace, rc_timestamp);
CREATE INDEX rc_ip ON recentchanges(rc_ip);

CREATE TABLE watchlist (
	wl_user				INTEGER NOT NULL
						REFERENCES "user"(user_id)
						ON DELETE CASCADE,
	wl_namespace			INTEGER DEFAULT 0 NOT NULL,
	wl_title			VARCHAR(255) NOT NULL,
	wl_notificationtimestamp	TIMESTAMP WITH TIME ZONE DEFAULT NULL
);
CREATE UNIQUE INDEX wl_user_namespace_title ON watchlist
	(wl_user, wl_namespace, wl_title);
CREATE INDEX wl_namespace_title ON watchlist(wl_namespace, wl_title);

--
-- Used by texvc math-rendering extension to keep track
-- of previously-rendered items.
--
CREATE TABLE math (
	math_inputhash			BYTEA NOT NULL UNIQUE,
	math_outputhash			BYTEA NOT NULL,
	math_html_conservativeness	INTEGER NOT NULL,
	math_html			TEXT,
	math_mathml			TEXT
);

-- Do the following if you have installed tsearch2 properly to get search to work
-- its still experimental though

CREATE TABLE /*$wgDBprefix*/searchindex (
        -- Key to page_id
        si_page INTEGER NOT NULL,

        -- Munged version of title
--      si_title VARCHAR(255) NOT NULL default '',

        -- Munged version of body text
--      si_text TEXT NOT NULL default '',

        si_title tsvector,

        si_text tsvector,

        PRIMARY KEY (si_page)
);

CREATE INDEX si_text_idx ON searchindex USING gist(si_text);
CREATE INDEX si_title_idx ON searchindex USING gist(si_title);


--
-- Recognized interwiki link prefixes
--
CREATE TABLE interwiki (
  iw_prefix	VARCHAR(32) NOT NULL UNIQUE,
  iw_url	VARCHAR(127) NOT NULL,
  iw_local	INTEGER NOT NULL,
  iw_trans	INTEGER DEFAULT 0 NOT NULL
);

CREATE TABLE querycache (
	qc_type		VARCHAR(32) NOT NULL,
	qc_value	INTEGER DEFAULT 0 NOT NULL,
	qc_namespace	INTEGER DEFAULT 0 NOT NULL,
	qc_title	VARCHAR(255)
);
CREATE INDEX querycache_type_value ON querycache(qc_type, qc_value);

--
-- For a few generic cache operations if not using Memcached
--
CREATE TABLE objectcache (
	keyname		CHAR(255) DEFAULT '',
	value		text,
	exptime		TIMESTAMP WITH TIME ZONE
);
CREATE UNIQUE INDEX oc_keyname_idx ON objectcache(keyname);
CREATE INDEX oc_exptime_idx ON objectcache(exptime);

CREATE TABLE "validate" (
	val_user	INTEGER DEFAULT 0 NOT NULL,
	val_page	INTEGER DEFAULT 0 NOT NULL,
	val_revision	INTEGER DEFAULT 0 NOT NULL,
	val_type	INTEGER DEFAULT 0 NOT NULL,
	val_value	INTEGER DEFAULT 0,
	val_comment	VARCHAR(255),
	val_ip		VARCHAR(20)
);
CREATE INDEX val_user ON "validate" (val_user,val_revision);

CREATE TABLE logging (
  log_type		VARCHAR(10) NOT NULL,
  log_action		VARCHAR(10) NOT NULL,
  log_timestamp		TIMESTAMP WITH TIME ZONE NOT NULL,
  log_user		INTEGER REFERENCES "user"(user_id),
  log_namespace		INTEGER,
  log_title		VARCHAR(255) NOT NULL,
  log_comment		VARCHAR(255),
  log_params		TEXT
);
CREATE INDEX logging_type_name ON logging(log_type, log_timestamp);
CREATE INDEX logging_user_time ON logging(log_user, log_timestamp);
CREATE INDEX logging_page_time ON logging(log_namespace, log_title, log_timestamp);

-- Hold group name and description
--CREATE TABLE /*$wgDBprefix*/groups (
--  gr_id int(5) unsigned NOT NULL auto_increment,
--  gr_name varchar(50) NOT NULL default '',
--  gr_description varchar(255) NOT NULL default '',
--  gr_rights tinyblob,
--  PRIMARY KEY  (gr_id)
--
--) TYPE=InnoDB;

--CREATE OR REPLACE PROCEDURE add_user_right (name VARCHAR, new_right VARCHAR) AS
--	user_id		"user".user_id%TYPE;;
--	user_is_missing	EXCEPTION;;
--BEGIN
--	SELECT user_id INTO user_id FROM "user" WHERE user_name = name;;
--	INSERT INTO user_groups (ug_user, ug_group) VALUES(user_id, new_right);;
--EXCEPTION
--	WHEN NO_DATA_FOUND THEN
--		DBMS_OUTPUT.PUT_LINE('The specified user does not exist.');;
--END add_user_right;;
--;

--CREATE OR REPLACE PROCEDURE add_interwiki (prefix VARCHAR, url VARCHAR, is_local INTEGER) AS
--BEGIN
--	INSERT INTO interwiki (iw_prefix, iw_url, iw_local) VALUES(prefix, url, is_local);;
--END add_interwiki;;
--;



CREATE OR REPLACE FUNCTION add_interwiki (text, text, integer) RETURNS INTEGER AS '
INSERT INTO interwiki (iw_prefix, iw_url, iw_local) VALUES ($1,$2,$3);;
SELECT 1;;
' LANGUAGE 'sql'
;


